io = require("socket.io").listen(8181)
sanitize = require("sanitizer").sanitize
sys = require("sys")

UserNames = []
Connections = []
Messages = []

PushMessage = (name, args...) ->
	Messages.push { Type: name, Args: args }
	if Messages.length > 40 then Messages = Messages.slice(1)

GenUsername = (basename) ->
	if not basename
		basename = "Anon" + Math.floor(Math.random() * 10000)
	name = basename
	idx = 0
	while (UserNames.indexOf(name) != -1)
		idx++
		name = basename + "(" + idx + ")"
	return name

io.sockets.on "connection", (socket) ->
	username = GenUsername()
	UserNames.push username
	Connections.push socket
	
	socket.on "Chat", (message) =>
		clean = sanitize(message)
		io.sockets.emit "Chat", username, clean
		PushMessage "Chat", username, clean
		
	socket.on "Nick", (name) =>
		old = username
		username = GenUsername sanitize(name)
		UserNames[UserNames.indexOf old] = username
		
		socket.emit "Identity", username
		io.sockets.emit "UserList", UserNames
		msg = "'#{ old }' is now known as '#{ username }'"
		io.sockets.emit "Server", msg
		PushMessage "Server", msg
		
	socket.on "PM", (to, message) =>
		index = UserNames.indexOf to
		if index == -1
			socket.emit "Server", "No user by the name of '#{ to }'"
			return
		cleaned = sanitize(message)
		Connections[index].emit "PM", username, to, cleaned
		socket.emit "PM", username, to, cleaned

	socket.on "Help", () =>
		HelpText = "
		<ul>
			<li>
				<b>/pm &lt;name&gt; &lt;message&gt;</b> to send a private message
			</li>
			<li>
				<b>/nick &lt;name&gt;</b> to change your name
			</li>
		</ul>"
		
		socket.emit("Help", HelpText);
	
	socket.on "disconnect", () =>
		index = UserNames.indexOf username
		Connections.splice index, 1
		UserNames.splice index, 1
		
		io.sockets.emit "Dropped", username
		PushMessage "Dropped", username
		io.sockets.emit "UserList", UserNames
	
	for msg in Messages
		socket.emit msg.Type, msg.Args...

	socket.emit "Identity", username
	socket.broadcast.emit "Joined", username
	PushMessage "Joined", username
	io.sockets.emit "UserList", UserNames
	socket.emit "Server", "Welcome to the chat '#{ username }'. Type /help to see a list of commands."
	